if select(6, GetAddOnInfo("PitBull_" .. (debugstack():match("[d%.][d%.][O%.]ns\\PitBull\\Modules\\(.-)\\") or ""))) ~= "MISSING" then return end

local VERSION = tonumber(("$Revision: 39938 $"):match("%d+"))

local PitBull = PitBull
local PitBull_PvPIcon = PitBull:NewModule("PvPIcon", "AceEvent-2.0")
local self = PitBull_PvPIcon
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-06-14 17:12:00 -0400 (Thu, 14 Jun 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end
PitBull.desc = "Show an icon on the unit frame when the unit is in PvP mode."

-- localization placeholder
local L = PitBull:L("PitBull-PvPIcon")

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame

PitBull_PvPIcon:RegisterPitBullChildFrames('pvpIcon')

function PitBull_PvPIcon:OnInitialize()
	self.db = PitBull:AcquireDBNamespace("PvPIcon")
	PitBull:RegisterDefaults("PvPIcon", "profile", {
		groups = {
			['*'] = {},
		},
	})
end

function PitBull_PvPIcon:OnEnable()
	self:RegisterEvent("UPDATE_FACTION", "Update")
	self:RegisterEvent("PLAYER_FLAGS_CHANGED", "Update")
	self:RegisterEvent("UNIT_FACTION", "Update")
end

function PitBull_PvPIcon:Update(unit, noPet)
	if not unit then
		unit = "player"
	end
	
	for frame in PitBull:IterateUnitFramesForUnit(unit) do
		if self.db.profile.groups[frame.group].ignore then
			if frame.pvpIcon then
				frame.pvpIcon = delFrame(frame.pvpIcon)
				PitBull:UpdateLayout(frame)
			end
			return
		end
		
		local has = false
		
		if UnitIsPVPFreeForAll(unit) then
			has = 'FFA'
		else
			local faction = UnitFactionGroup(unit)
			if faction and UnitIsPVP(unit) then
				has = faction
			end
		end
		
		if has then
			if not frame.pvpIcon then
				frame.pvpIcon = newFrame("Texture", frame.overlay, "ARTWORK")
				frame.pvpIcon:Hide()
				PitBull:UpdateLayout(frame)
			end
			frame.pvpIcon:SetTexture("Interface\\TargetingFrame\\UI-PVP-" .. has)
		else
			if frame.pvpIcon then
				frame.pvpIcon = delFrame(frame.pvpIcon)

				PitBull:UpdateLayout(frame)
			end
		end
	end
	
	if not noPet then
		if unit == "player" then
			self:Update("pet")
		else
			local num = unit:match("^party(%d)$")
			if num then
				self:Update("partypet" .. num)
			end
		end
	end
end

function PitBull_PvPIcon:OnUpdateFrame(unit, frame)
	self:Update(unit, true)
end

function PitBull_PvPIcon:OnClearUnitFrame(unit, frame)
	if frame.pvpIcon then
		frame.pvpIcon = delFrame(frame.pvpIcon)
	end
end

function PitBull_PvPIcon:OnUnknownLayout(unit, frame)
	frame.pvpIcon:SetPoint("CENTER", frame, "TOPRIGHT", 0, 0)
	frame.pvpIcon:SetWidth(27)
	frame.pvpIcon:SetHeight(27)
end

local function getEnabled(group)
	return not PitBull_PvPIcon.db.profile.groups[group].ignore
end
local function setEnabled(group, value)
	value = not value
	PitBull_PvPIcon.db.profile.groups[group].ignore = value
	if value then
		for unit,frame in PitBull:IterateUnitFramesByGroup(group) do
			if frame.pvpIcon then
				frame.pvpIcon = delFrame(frame.pvpIcon)
			end
		end
	else
		for unit in PitBull:IterateUnitFramesByGroup(group) do
			PitBull_PvPIcon:Update(unit)
		end
	end
end
PitBull_PvPIcon:RegisterPitBullOptionsMethod(function(group)
	return {
		type = "toggle",
		name = L["Enable PvP icon"],
		desc = L["Enables the PvP icon, indicating if this unit is flagged for PvP."],
		get = getEnabled,
		set = setEnabled,
		passValue = group,
	}
end)

